// gemc headers
#include "momControls.h"
#include "string_utilities.h"

// G4 headers
#include "G4ParticleTable.hh"



momControls::momControls(goptions *Opts, QWidget *parent, string type)
{
	G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();   ///< Geant4 Particle Table
	
	// vector of string - filled from the various option
	vector<string> values;
	string units;
	double arg_mom,  arg_theta,  arg_phi;
	double arg_rmom, arg_rtheta, arg_rphi;
	string pname;
	
	// Getting momentum from option value
	if(type == "Primary") values    = get_info(Opts->optMap["BEAM_P"].args);
	if(type == "Lumi1")   values    = get_info(Opts->optMap["LUMI_P"].args);
	if(type == "Lumi2")   values    = get_info(Opts->optMap["LUMI2_P"].args);
	pname     = TrimSpaces(values[0]);
	arg_mom   = get_number(values[1])/MeV;
	arg_theta = get_number(values[2])/deg;
	arg_phi   = get_number(values[3])/deg;
	
	
	if(type == "Primary") values    = get_info(Opts->optMap["SPREAD_P"].args);
	if(type == "Lumi1")   values    = get_info(Opts->optMap["LUMI_SPREAD_P"].args);
	if(type == "Lumi2")   values    = get_info(Opts->optMap["LUMI2_SPREAD_P"].args);
	arg_rmom   = get_number(values[0])/MeV;
	arg_rtheta = get_number(values[1])/deg;
	arg_rphi   = get_number(values[2])/deg;
	
	
	
	// check that particle is in particle table
	if(!particleTable->FindParticle(pname))
		cout << "  !! GUI Control: Particle " << pname << " not found in G4 table." << endl << endl;
	
	// filling drop down menu with all particles availabe
	beam_particle = new QComboBox;
	// most important particles on top
	beam_particle->addItem(pname.c_str());
	beam_particle->addItem("e-");
	beam_particle->addItem("proton");
	beam_particle->addItem("pi+");
	beam_particle->addItem("pi-");
	beam_particle->addItem("kaon+");
	beam_particle->addItem("kaon-");
	for(int i=0; i<particleTable->entries(); i++)
		beam_particle->addItem(particleTable->GetParticleName(i).c_str());
	connect ( beam_particle  , SIGNAL( currentIndexChanged(int) ), parent, SLOT( set_beam_values() ) );
	
	// Particle Type Layout
	QHBoxLayout *beam_particleLayout = new QHBoxLayout;
	beam_particleLayout->addSpacing(40);
	beam_particleLayout->addWidget(new QLabel(tr("Particle Type:")));
	beam_particleLayout->addWidget(beam_particle);
	beam_particleLayout->addSpacing(80);
	
	// Labels at top of sliders
	QHBoxLayout *explanaLayout = new QHBoxLayout;
	explanaLayout->addSpacing(120);
	explanaLayout->addWidget(new QLabel(tr("Value")));
	explanaLayout->addSpacing(190);
	explanaLayout->addWidget(new QLabel(tr("Dispersion")));
	explanaLayout->addSpacing(75);
	explanaLayout->addWidget(new QLabel(tr("Zoom")));
	
	
	
	
	mom_sliders_nticks = 10001;   // how many ticks
	
	// momentum
	mom_slider_min   = 0;                                            // starting minimum in MeV
	mom_slider_max   = get_max_ten_from_value(arg_mom*2, arg_mom*3); // starting maximum is arg_mom rounded up
	
	momentum_slider = new QSlider(Qt::Horizontal);
	momentum_slider->setRange(0, mom_sliders_nticks-1);
	// the 0.5 is added because of the (int) truncation
	momentum_slider->setSliderPosition((int) ((arg_mom-mom_slider_min)/get_dmom()*(mom_sliders_nticks-1) + 0.5 ) );
	
	// momentum dispersion up to 2*momentum
	rmom_slider_max  = 2*mom_slider_max;
	rmomentum_slider = new QSlider(Qt::Horizontal);
	rmomentum_slider->setRange(0, mom_sliders_nticks-1);
	rmomentum_slider->setSliderPosition((int) (arg_rmom/rmom_slider_max*(mom_sliders_nticks-1) + 0.5 ) );
	
	
	QPushButton *momPlus = new QPushButton("+");
	QPushButton *momMinu = new QPushButton("-");
	momPlus -> setFont(QFont("Times-Roman", 20, QFont::Bold));
	momMinu -> setFont(QFont("Times-Roman", 20, QFont::Bold));
	momPlus -> setMaximumSize(20, 20);
	momMinu -> setMaximumSize(20, 20);
	
	connect ( momentum_slider  , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	connect ( rmomentum_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	connect ( momPlus ,          SIGNAL(         clicked() ), this,   SLOT( increaseMomZoon() ) );
	connect ( momPlus ,          SIGNAL(         clicked() ), parent, SLOT( set_beam_values() ) );
	connect ( momMinu ,          SIGNAL(         clicked() ), this,   SLOT( decreaseMomZoon() ) );
	connect ( momMinu ,          SIGNAL(         clicked() ), parent, SLOT( set_beam_values() ) );
	
	QHBoxLayout *momentumLayout = new QHBoxLayout;
	momentumLayout->addWidget(new QLabel(tr("p")));
	momentumLayout->addSpacing(5);
	momentumLayout->addWidget(momentum_slider);
	momentumLayout->addWidget(rmomentum_slider);
	momentumLayout->addWidget(momPlus);
	momentumLayout->addWidget(momMinu);
	
	
	// theta
	theta_slider_range = 3600;
	theta_slider_scale = 1.0/20;
	
	theta_slider = new QSlider(Qt::Horizontal);
	theta_slider->setRange(0, theta_slider_range);
	theta_slider->setSliderPosition( (int) (arg_theta/theta_slider_scale));
	rtheta_slider = new QSlider(Qt::Horizontal);
	rtheta_slider->setRange(0, theta_slider_range);
	rtheta_slider->setSliderPosition((int) (arg_rtheta/theta_slider_scale));
	connect ( theta_slider  , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	connect ( rtheta_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	
	QHBoxLayout *thetaLayout = new QHBoxLayout;
	thetaLayout->addWidget(new QLabel(QString::fromLocal8Bit("θ")));
	thetaLayout->addWidget(theta_slider);
	thetaLayout->addWidget(rtheta_slider);
	
	
	// phi
	phi_slider_range = 3600;
	phi_slider_scale = 1.0/10;
	
	phi_slider = new QSlider(Qt::Horizontal);
	phi_slider->setRange(0, phi_slider_range);
	phi_slider->setSliderPosition((int) (arg_phi/phi_slider_scale));
	rphi_slider = new QSlider(Qt::Horizontal);
	rphi_slider->setRange(0, phi_slider_range);
	rphi_slider->setSliderPosition((int) (2*arg_rphi/phi_slider_scale));
	connect ( phi_slider  , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	connect ( rphi_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_beam_values() ) );
	
	
	QHBoxLayout *phiLayout = new QHBoxLayout;
	phiLayout->addWidget(new QLabel(QString::fromLocal8Bit("ϕ")));
	phiLayout->addSpacing(5);
	phiLayout->addWidget(phi_slider);
	phiLayout->addWidget(rphi_slider);
	
	
	
	momentumGroup = new QGroupBox(tr(""));
	QVBoxLayout *mLayout = new QVBoxLayout;
	mLayout->addLayout(beam_particleLayout);
	mLayout->addSpacing(10);
	mLayout->addLayout(explanaLayout);
	mLayout->addLayout(momentumLayout);
	mLayout->addLayout(thetaLayout);
	mLayout->addLayout(phiLayout);
	momentumGroup->setLayout(mLayout);
	
	
	if(type == "Primary")
		momentumGroup->setStyleSheet(" * { background-color: rgb(220, 230, 240);} QLabel {background-color: transparent; }");
	
	if(type == "Lumi1")
		momentumGroup->setStyleSheet(" * { background-color: rgb(220, 245, 235);} QLabel {background-color: transparent; }");
	
	if(type == "Lumi2")
		momentumGroup->setStyleSheet(" * { background-color: rgb(195, 240, 195);} QLabel {background-color: transparent; }");
	
	
	// Momentum Readout Group
	mroGroup = new QGroupBox(tr("Beam"));
	mroGroup->setMaximumHeight(125);
	
	QHBoxLayout *momentum_roLayout_title = new QHBoxLayout;
	momentum_roLayout_title->addSpacing(40);
	momentum_roLayout_title->addWidget(new QLabel(tr("Gen. Range")));
	momentum_roLayout_title->addSpacing(30);
	momentum_roLayout_title->addWidget(new QLabel(tr("Slider Range")));
	
	
	QHBoxLayout *momentum_roLayout = new QHBoxLayout;
	momentum_roLayout->addWidget(new QLabel(tr("p:")));
	momentum_ro = new QLabel(tr(""));
	momentum_roLayout->addWidget(momentum_ro);
	momentum_roLayout->addSpacing(20);
	rmomentum_ro = new QLabel(tr(""));
	momentum_roLayout->addWidget(rmomentum_ro);
	
	QHBoxLayout *theta_roLayout = new QHBoxLayout;
	theta_roLayout->addWidget(new QLabel(QString::fromLocal8Bit("θ:")));
	theta_ro = new QLabel(tr(""));
	theta_roLayout->addWidget(theta_ro);
	
	QHBoxLayout *phi_roLayout = new QHBoxLayout;
	phi_roLayout->addWidget(new QLabel(QString::fromLocal8Bit("ϕ:")));
	phi_ro = new QLabel(tr(""));
	phi_roLayout->addWidget(phi_ro);
	
	QVBoxLayout *mroLayout = new QVBoxLayout;
	mroLayout->addLayout(momentum_roLayout_title);
	mroLayout->addLayout(momentum_roLayout);
	mroLayout->addLayout(theta_roLayout);
	mroLayout->addLayout(phi_roLayout);
	mroGroup->setLayout(mroLayout);
	
	if(type == "Primary")
		mroGroup->setStyleSheet(" * { background-color: rgb(220, 230, 240);} QLabel {background-color: transparent; }");
	
	if(type == "Lumi1")
		mroGroup->setStyleSheet(" * { background-color: rgb(215, 220, 215);} QLabel {background-color: transparent; }");
	
	if(type == "Lumi2")
		mroGroup->setStyleSheet(" * { background-color: rgb(195, 240, 195);} QLabel {background-color: transparent; }");
	
}

void momControls::set_momentum_ro()
{
	double mom  = get_mom_from_slider();
	double rmom = get_rmom_from_slider();
	
	QString mro = stringify(mom).c_str();
	mro.append(QString::fromUtf8(" ± "));
	mro.append(stringify(rmom).c_str());
	mro.append(" MeV");
	momentum_ro->setText(mro);
}

void momControls::set_rmomentum_ro()
{
	QString mro = stringify(mom_slider_min).c_str();
	mro.append(QString::fromUtf8(" - "));
	mro.append(stringify(mom_slider_max).c_str());
	mro.append(" MeV");
	rmomentum_ro->setText(mro);
}

void momControls::set_theta_ro()
{
	double the  = get_theta_from_slider();
	double rthe = get_rtheta_from_slider();
	
	QString tro = stringify(the).c_str();
	tro.append(QString::fromUtf8(" ± "));
	tro.append(stringify(rthe).c_str());
	tro.append(" deg");
	theta_ro->setText(tro);
}

void momControls::set_phi_ro()
{
	double phi  = get_phi_from_slider();
	double rphi = get_rphi_from_slider();
	
	QString pro = stringify(phi).c_str();
	pro.append(QString::fromUtf8(" ± "));
	pro.append(stringify(rphi).c_str());
	pro.append(" deg");
	phi_ro->setText(pro);
}













