// gemc headers
#include "HitProcess.h"

HitProcess *getHitProcess(map<string, HitProcess_Factory> *hitProcessMap, string HCname)
{
	if(HCname == "no")
		return NULL;
	
	if(hitProcessMap->find(HCname) == hitProcessMap->end())
	{
		cout << endl << "  !!! Error: >" << HCname << "< NOT FOUND IN  ProcessHit Map. Exiting" << endl;
		exit(0);
		return NULL;
	}
	return (*hitProcessMap)[HCname]();
}

// returns the list of Hit Factories registered
set<string> getListOfHitProcessHit(map<string, HitProcess_Factory> hitProcessMap)
{
	set<string> listF;
	
	for(map<string, HitProcess_Factory>::iterator it = hitProcessMap.begin(); it != hitProcessMap.end(); it++)
		listF.insert(it->first);
	
	return listF;
}


// - integrateRaw: returns geant4 raw information integrated over the hit
map<string, double> HitProcess::integrateRaw(MHit* aHit, int hitn, bool WRITEBANK)
{
	map<string, double> raws;
	
	trueInfos tInfos(aHit);
	
	
	if(WRITEBANK)
	{
		raws["hitn"]    = hitn;
		raws["pid"]     = (double) aHit->GetPID();
		raws["mpid"]    = (double) aHit->GetmPID();
		raws["tid"]     = (double) aHit->GetTId();
		raws["mtid"]    = (double) aHit->GetmTrackId();
		raws["otid"]    = (double) aHit->GetoTrackId();
		raws["trackE"]  = aHit->GetE();
		raws["totEdep"] = tInfos.eTot;
		raws["avg_x"]   = tInfos.x;
		raws["avg_y"]   = tInfos.y;
		raws["avg_z"]   = tInfos.z;
		raws["avg_lx"]  = tInfos.lx;
		raws["avg_ly"]  = tInfos.ly;
		raws["avg_lz"]  = tInfos.lz;
		raws["avg_t"]   = tInfos.time;
		raws["px"]      = aHit->GetMom().getX();
		raws["py"]      = aHit->GetMom().getY();
		raws["pz"]      = aHit->GetMom().getZ();
		raws["vx"]      = aHit->GetVert().getX();
		raws["vy"]      = aHit->GetVert().getY();
		raws["vz"]      = aHit->GetVert().getZ();
		raws["mvx"]     = aHit->GetmVert().getX();
		raws["mvy"]     = aHit->GetmVert().getY();
		raws["mvz"]     = aHit->GetmVert().getZ();
	}
	return raws;
}



map< string, vector <double> > HitProcess::allRaws(MHit*, int hitn)
{
	map< string, vector <double> > allRaws;
	
	return allRaws;
}



map< double, double >  HitProcess :: signalVT(MHit* aHit, int hitn)
{
	// HARDCODING TIME RESOLUTION HERE
	double tres = 0.1*ns;
	
	
	map< double, double > VT;
	sensitiveID SID = aHit->GetSDID();
	
	// setting signal parameters
	par[0] = SID.delay;
	par[1] = SID.riseTime;
	par[2] = SID.fallTime;
	par[3] = SID.mvToMeV;
	
	vector<G4double>  Edep  = aHit->GetEdep();
	vector<G4double>  times = aHit->GetTime();
	unsigned nsteps = Edep.size();	
	
	double tmin = 100000;
	double tmax = 0;
	
	
	// finding min, max
	for(unsigned int s=0; s<nsteps; s++)
	{
		if(tmin > times[s]) tmin = times[s];
		if(tmax < times[s]) tmax = times[s];
	//	cout << " step " << s+1 << " time " << times[s] << "  tmin " << tmin << "  tmax " << tmax << endl;
	}
	
	// adding 1/2 DELAY to tmin
	tmin = floor(tmin + 0.5*SID.delay);
	
	// adding a whole signal to tmax
	tmax = floor(tmax + SID.delay + SID.riseTime + 2*SID.fallTime);
	
	unsigned int nt = (int) (tmax - tmin)/tres;


	//cout <<  "  tmin " << tmin << "  tmax " << tmax << "  nt " << nt << endl;
	
	for(unsigned ti=0; ti<nt; ti++)
	{
		// local time
		double localT = tmin + ti*tres;
		double totV   = 0;
		
		// v signal at this time is sum of all signals at each step
		for(unsigned int s=0; s<nsteps; s++)
		{			
			totV += DGauss(localT, par, Edep[s], times[s]);
			
//			cout << " local time " << localT << "  step " << s + 1 << " time " << times[s]
//			<< "  Edep " << Edep[s] << "  signal " << DGauss(localT, par, Edep[s], times[s]) << endl;
		}
		
		VT[localT] = totV + SID.pedestal;
	}
	
	
//	for(map<double, double>::iterator it = VT.begin(); it != VT.end(); it++)
//		cout << "  time " << it->first << "   VT " << it->second << endl;
	
	aHit->setSignal(VT);
	
	return VT;
}



trueInfos::trueInfos(MHit* aHit)
{
	eTot = 0;
	time = 0;
	x = y = z = lx = ly = lz = 0;

	// getting vectors of energy deposited, positions and times
	// nsteps is the size

	vector<G4double> Edep       = aHit->GetEdep();
	vector<G4ThreeVector> pos   = aHit->GetPos();
	vector<G4ThreeVector> Lpos  = aHit->GetLPos();
	vector<G4double>      times = aHit->GetTime();

	nsteps = Edep.size();
	for(unsigned int s=0; s<nsteps; s++)
		eTot += Edep[s];

	// if energy deposited, averaging quantities over the hit
	// weighted by energy deposited
	if(eTot)
	{
		for(unsigned int s=0; s<nsteps; s++)
		{
			x    +=  pos[s].x()*Edep[s];
			y    +=  pos[s].y()*Edep[s];
			z    +=  pos[s].z()*Edep[s];
			lx   += Lpos[s].x()*Edep[s];
			ly   += Lpos[s].y()*Edep[s];
			lz   += Lpos[s].z()*Edep[s];
			time += times[s]*Edep[s];
		}
		x    = x/eTot;
		y    = y/eTot;
		z    = z/eTot;
		lx   = lx/eTot;
		ly   = ly/eTot;
		lz   = lz/eTot;
		time = time/eTot;
	}
	else
	{
		for(unsigned int s=0; s<nsteps; s++)
		{
			x    +=  pos[s].x();
			y    +=  pos[s].y();
			z    +=  pos[s].z();
			lx   += Lpos[s].x();
			ly   += Lpos[s].y();
			lz   += Lpos[s].z();
			time += times[s];
		}
		x    = x/nsteps;
		y    = y/nsteps;
		z    = z/nsteps;
		lx   = lx/nsteps;
		ly   = ly/nsteps;
		lz   = lz/nsteps;
		time = time/nsteps;
	}
}



